"""Evo device class for Ariston module."""
from __future__ import annotations

import logging
from datetime import datetime
from typing import Optional

from .const import (
    EvoPlantMode,
    EvoDeviceProperties,
    MedDeviceSettings,
    PlantData,
    WaterHeaterMode,
)
from .evo_lydos_device import AristonEvoLydosDevice

_LOGGER = logging.getLogger(__name__)


class AristonEvoDevice(AristonEvoLydosDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""
        return PlantData.Med

    @property
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""
        return MedDeviceSettings.MED_ANTILEGIONELLA_ON_OFF

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return "Dhw"

    @property
    def water_heater_mode(self) -> type[WaterHeaterMode]:
        """Return the water heater mode class"""
        return EvoPlantMode

    @property
    def max_setpoint_temp(self) -> str:
        return MedDeviceSettings.MED_MAX_SETPOINT_TEMPERATURE

    @property
    def water_heater_eco_value(self) -> Optional[int]:
        """Get water heater eco value"""
        return self.data.get(EvoDeviceProperties.ECO, None)

    @property
    def rm_tm_value(self) -> Optional[str]:
        """Get remaining time value"""
        return self.data.get(EvoDeviceProperties.RM_TM, None)

    @property
    def rm_tm_in_minutes(self) -> int:
        """Get remaining time value in minutes"""
        rm_tm = self.rm_tm_value
        if rm_tm is None:
            return -1
        time = datetime.strptime(rm_tm, "%H:%M:%S")
        return time.hour * 60 + time.minute

    @property
    def water_heater_power_option_value(self) -> Optional[bool]:
        """Get water heater power option value"""
        return self.data.get(EvoDeviceProperties.PWR_OPT, None)

    @property
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        return self.plant_settings.get(
            MedDeviceSettings.MED_MAX_SETPOINT_TEMPERATURE_MIN, None
        )

    @property
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        return self.plant_settings.get(
            MedDeviceSettings.MED_MAX_SETPOINT_TEMPERATURE_MAX, None
        )

    def set_eco_mode(self, eco_mode: bool):
        """Set water heater eco_mode"""
        self.api.set_evo_eco_mode(self.gw, eco_mode)
        self.data[EvoDeviceProperties.ECO] = eco_mode

    async def async_set_eco_mode(self, eco_mode: bool):
        """Async set water heater eco_mode"""
        await self.api.async_set_evo_eco_mode(self.gw, eco_mode)
        self.data[EvoDeviceProperties.ECO] = eco_mode

    def set_water_heater_operation_mode(self, operation_mode: str):
        """Set water heater operation mode"""
        self.api.set_evo_mode(self.gw, EvoPlantMode[operation_mode])
        self.data[EvoDeviceProperties.MODE] = EvoPlantMode[operation_mode].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str):
        """Async set water heater operation mode"""
        await self.api.async_set_evo_mode(self.gw, EvoPlantMode[operation_mode])
        self.data[EvoDeviceProperties.MODE] = EvoPlantMode[operation_mode].value

    def set_water_heater_temperature(self, temperature: float):
        """Set water heater temperature"""
        self.api.set_evo_temperature(self.gw, temperature)
        self.data[EvoDeviceProperties.REQ_TEMP] = temperature

    async def async_set_water_heater_temperature(self, temperature: float):
        """Async set water heater temperature"""
        await self.api.async_set_evo_temperature(self.gw, temperature)
        self.data[EvoDeviceProperties.REQ_TEMP] = temperature



"""Velis device class for Ariston module."""
from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from typing import Any, Optional

from .ariston_api import AristonAPI
from .const import (
    CustomDeviceFeatures,
    DeviceFeatures,
    PlantData,
    VelisDeviceProperties,
    WaterHeaterMode,
)
from .device import AristonDevice

_LOGGER = logging.getLogger(__name__)


class AristonVelisDevice(AristonDevice, ABC):
    """Class representing a physical device, it's state and properties."""

    def __init__(
        self,
        api: AristonAPI,
        attributes: dict[str, Any],
    ) -> None:
        super().__init__(api, attributes)
        self.plant_settings: dict[str, Any] = dict()

    @property
    @abstractmethod
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""

    @property
    @abstractmethod
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""

    @property
    @abstractmethod
    def water_heater_mode(self) -> type[WaterHeaterMode]:
        """Return the water heater mode class"""

    @property
    @abstractmethod
    def max_setpoint_temp(self) -> str:
        """Final string to get max setpoint temperature"""

    def update_state(self) -> None:
        """Update the device states from the cloud"""
        self.data = self.api.get_velis_plant_data(self.plant_data, self.gw)

    async def async_update_state(self) -> None:
        """Async update the device states from the cloud"""
        self.data = await self.api.async_get_velis_plant_data(self.plant_data, self.gw)

    def update_settings(self) -> None:
        """Get device settings wrapper"""
        self.plant_settings = self.api.get_velis_plant_settings(
            self.plant_data, self.gw
        )

    async def async_update_settings(self) -> None:
        """Get device settings wrapper"""
        self.plant_settings = await self.api.async_get_velis_plant_settings(
            self.plant_data, self.gw
        )

    def set_power(self, power: bool):
        """Set water heater power"""
        self.api.set_velis_power(self.plant_data, self.gw, power)
        self.data[VelisDeviceProperties.ON] = power

    async def async_set_power(self, power: bool) -> None:
        """Async set water heater power"""
        await self.api.async_set_velis_power(self.plant_data, self.gw, power)
        self.data[VelisDeviceProperties.ON] = power

    @property
    def water_anti_leg_value(self) -> Optional[bool]:
        """Get water heater anti-legionella value"""
        return self.plant_settings.get(self.anti_legionella_on_off, None)

    def set_antilegionella(self, anti_leg: bool):
        """Set water heater anti-legionella"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.anti_legionella_on_off,
            1.0 if anti_leg else 0.0,
            1.0 if self.plant_settings[self.anti_legionella_on_off] else 0.0,
        )
        self.plant_settings[self.anti_legionella_on_off] = anti_leg

    async def async_set_antilegionella(self, anti_leg: bool):
        """Async set water heater anti-legionella"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.anti_legionella_on_off,
            1.0 if anti_leg else 0.0,
            1.0 if self.plant_settings[self.anti_legionella_on_off] else 0.0,
        )
        self.plant_settings[self.anti_legionella_on_off] = anti_leg

    @property
    def water_heater_mode_operation_texts(self) -> list[str]:
        """Get water heater operation mode texts"""
        return [flag.name for flag in self.water_heater_mode]

    @property
    def water_heater_mode_options(self) -> list[int]:
        """Get water heater operation options"""
        return [flag.value for flag in self.water_heater_mode]

    def set_max_setpoint_temp(self, max_setpoint_temp: float):
        """Set water heater maximum setpoint temperature"""
        self.api.set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.max_setpoint_temp,
            max_setpoint_temp,
            self.plant_settings[self.max_setpoint_temp],
        )
        self.plant_settings[self.max_setpoint_temp] = max_setpoint_temp

    async def async_set_max_setpoint_temp(self, max_setpoint_temp: float):
        """Async set water heater maximum setpoint temperature"""
        await self.api.async_set_velis_plant_setting(
            self.plant_data,
            self.gw,
            self.max_setpoint_temp,
            max_setpoint_temp,
            self.plant_settings[self.max_setpoint_temp],
        )
        self.plant_settings[self.max_setpoint_temp] = max_setpoint_temp

    def get_features(self) -> None:
        """Get device features wrapper"""
        super().get_features()
        self.custom_features[CustomDeviceFeatures.HAS_DHW] = True
        self.features[DeviceFeatures.DHW_MODE_CHANGEABLE] = True
        self.update_settings()

    async def async_get_features(self) -> None:
        """Async get device features wrapper"""
        await super().async_get_features()
        self.custom_features[CustomDeviceFeatures.HAS_DHW] = True
        self.features[DeviceFeatures.DHW_MODE_CHANGEABLE] = True
        await self.async_update_settings()

    @property
    @abstractmethod
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        raise NotImplementedError

    @property
    def water_heater_maximum_setpoint_temperature(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature value"""
        return self.plant_settings.get(self.max_setpoint_temp, None)

    @property
    def water_heater_minimum_temperature(self) -> float:
        """Get water heater minimum temperature"""
        return 40.0

    @property
    def water_heater_maximum_temperature(self) -> Optional[float]:
        """Get water heater maximum temperature"""
        return self.water_heater_maximum_setpoint_temperature

    @property
    def water_heater_temperature_step(self) -> int:
        """Get water heater temperature step"""
        return 1

    @property
    def water_heater_temperature_decimals(self) -> int:
        """Get water heater temperature decimals"""
        return 0

    @property
    def water_heater_temperature_unit(self) -> str:
        """Get water heater temperature unit"""
        return "°C"

    @property
    def water_heater_mode_value(self) -> Optional[int]:
        """Get water heater mode value"""
        return self.data.get(VelisDeviceProperties.MODE, None)

    @property
    def water_heater_power_value(self) -> Optional[bool]:
        """Get water heater power value"""
        return self.data.get(VelisDeviceProperties.ON, None)



"""Lydos hybrid device class for Ariston module."""
from __future__ import annotations

import logging
from typing import Optional

from .const import (
    ConsumptionTimeInterval,
    ConsumptionType,
    LydosPlantMode,
    PlantData,
    SeDeviceSettings,
    LydosDeviceProperties,
)
from .evo_lydos_device import AristonEvoLydosDevice

_LOGGER = logging.getLogger(__name__)


class AristonLydosHybridDevice(AristonEvoLydosDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def plant_data(self) -> PlantData:
        """Final string to get plant data"""
        return PlantData.Se

    @property
    def anti_legionella_on_off(self) -> str:
        """Final string to get anti-legionella-on-off"""
        return SeDeviceSettings.SE_ANTILEGIONELLA_ON_OFF

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return "DhwHeatingPumpElec%2CDhwResistorElec"

    @property
    def water_heater_mode(self) -> type[LydosPlantMode]:
        """Return the water heater mode class"""
        return LydosPlantMode

    @property
    def max_setpoint_temp(self) -> str:
        return SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE

    @property
    def water_heater_maximum_setpoint_temperature_minimum(self) -> Optional[float]:
        """Get water heater maximum setpoint temperature minimum"""
        return self.plant_settings.get(
            SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE_MIN, None
        )

    @property
    def water_heater_maximum_setpoint_temperature_maximum(self) -> Optional[float]:
        """Get water heater maximum setpoint maximum temperature"""
        return self.plant_settings.get(
            SeDeviceSettings.SE_MAX_SETPOINT_TEMPERATURE_MAX, None
        )

    @property
    def electric_consumption_for_water_last_two_hours(self) -> int:
        """Get electric consumption for water last value"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_HEATING_PUMP_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    def set_water_heater_operation_mode(self, operation_mode: str):
        """Set water heater operation mode"""
        self.api.set_lydos_mode(self.gw, LydosPlantMode[operation_mode])
        self.data[LydosDeviceProperties.MODE] = LydosPlantMode[operation_mode].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str):
        """Async set water heater operation mode"""
        await self.api.async_set_lydos_mode(self.gw, LydosPlantMode[operation_mode])
        self.data[LydosDeviceProperties.MODE] = LydosPlantMode[operation_mode].value

    def set_water_heater_temperature(self, temperature: float):
        """Set water heater temperature"""
        self.api.set_lydos_temperature(self.gw, temperature)
        self.data[LydosDeviceProperties.REQ_TEMP] = temperature

    async def async_set_water_heater_temperature(self, temperature: float):
        """Async set water heater temperature"""
        await self.api.async_set_lydos_temperature(self.gw, temperature)
        self.data[LydosDeviceProperties.REQ_TEMP] = temperature



"""Device class for Ariston module."""
from __future__ import annotations

import datetime as dt
import logging
from abc import ABC, abstractmethod
from typing import Any, Optional

from .ariston_api import AristonAPI
from .const import (
    ConsumptionTimeInterval,
    ConsumptionType,
    DeviceFeatures,
    DeviceAttribute,
    GalevoDeviceAttribute,
    SystemType,
    VelisDeviceAttribute,
    WheType,
)

_LOGGER = logging.getLogger(__name__)


class AristonDevice(ABC):
    """Class representing a physical device, it's state and properties."""

    def __init__(
        self,
        api: AristonAPI,
        attributes: dict[str, Any],
    ) -> None:
        self.api = api
        self.attributes = attributes

        self.features: dict[str, Any] = dict()
        self.custom_features: dict[str, Any] = dict()
        self.consumptions_sequences: list[dict[str, Any]] = list()
        self.data: dict[str, Any] = dict()
        self.consumption_sequence_last_changed_utc: dt.datetime = (
            dt.datetime.utcfromtimestamp(0).replace(tzinfo=dt.timezone.utc)
        )
        self.gw: str = self.attributes.get(DeviceAttribute.GW, "")
        self.bus_errors_list: list[dict[str, Any]] = []

    @property
    @abstractmethod
    def consumption_type(self) -> str:
        """String to get consumption type"""

    def _get_consumptions_sequences(self) -> None:
        """Get consumption sequence"""
        self.consumptions_sequences = self.api.get_consumptions_sequences(
            self.gw,
            self.consumption_type,
        )

    async def _async_get_consumptions_sequences(self) -> None:
        """Async get consumption sequence"""
        self.consumptions_sequences = await self.api.async_get_consumptions_sequences(
            self.gw,
            self.consumption_type,
        )

    @property
    def system_type(self) -> SystemType:
        """Get device system type wrapper"""
        return SystemType(self.attributes.get(DeviceAttribute.SYS, SystemType.UNKNOWN))

    @property
    def whe_type(self) -> WheType:
        """Get device whe type wrapper"""
        return WheType(
            self.attributes.get(VelisDeviceAttribute.WHE_TYPE, WheType.Unknown)
        )

    @property
    def whe_model_type(self) -> int:
        """Get device whe model type wrapper"""
        return self.attributes.get(VelisDeviceAttribute.WHE_MODEL_TYPE, 0)

    @property
    def gateway(self) -> str:
        """Get device gateway wrapper"""
        return self.gw

    @property
    def has_metering(self) -> Optional[bool]:
        """Get device has metering wrapper"""
        return self.features.get(DeviceFeatures.HAS_METERING, None)

    @property
    def name(self) -> Optional[str]:
        """Get device name wrapper"""
        return self.attributes.get(DeviceAttribute.NAME, None)

    @property
    def dhw_mode_changeable(self) -> Optional[bool]:
        """Get device domestic hot water mode changeable wrapper"""
        return self.features.get(DeviceFeatures.DHW_MODE_CHANGEABLE, None)

    @property
    def serial_number(self) -> Optional[str]:
        """Get device serial number wrapper"""
        return self.attributes.get(DeviceAttribute.SN, None)

    @property
    def firmware_version(self) -> Optional[str]:
        """Get device firmware version wrapper"""
        return self.attributes.get(GalevoDeviceAttribute.FW_VER, None)
    
    @property
    def bus_errors(self) -> list[dict[str, Any]]:
        """Get bus errors list wrapper"""
        return self.bus_errors_list

    def get_features(self) -> None:
        """Get device features wrapper"""
        self.features = self.api.get_features_for_device(self.gw)

    async def async_get_features(self) -> None:
        """Async get device features wrapper"""
        features = await self.api.async_get_features_for_device(self.gw)
        if features is not None:
            self.features = features

    @property
    def water_heater_current_mode_text(self) -> str:
        """Get water heater current mode text"""
        mode = self.water_heater_mode_value
        if mode in self.water_heater_mode_options:
            index = self.water_heater_mode_options.index(mode)
            return self.water_heater_mode_operation_texts[index]
        return "UNKNOWN"

    @abstractmethod
    def update_state(self) -> None:
        """Update the device states from the cloud"""
        raise NotImplementedError

    @abstractmethod
    async def async_update_state(self) -> None:
        """Async update the device states from the cloud"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_current_temperature(self) -> Optional[float]:
        """Abstract method for get water heater current temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_minimum_temperature(self) -> float:
        """Abstract method for get water heater minimum temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_target_temperature(self) -> Optional[float]:
        """Abstract method for get water heater target temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_maximum_temperature(self) -> Optional[float]:
        """Abstract method for get water heater maximum temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_temperature_decimals(self) -> int:
        """Abstract method for get water heater temperature decimals"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_temperature_unit(self) -> str:
        """Abstract method for get water heater temperature unit"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_mode_operation_texts(self) -> list[str]:
        """Abstract method for get water heater operation texts"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_mode_options(self) -> list[int]:
        """Abstract method for get water heater mode options"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_mode_value(self) -> Optional[int]:
        """Abstract method for get water heater mode value"""
        raise NotImplementedError

    @abstractmethod
    def set_water_heater_temperature(self, temperature: float) -> None:
        """Abstract method for set water temperature"""
        raise NotImplementedError

    @abstractmethod
    async def async_set_water_heater_temperature(self, temperature: float) -> None:
        """Abstract method for async set water temperature"""
        raise NotImplementedError

    @property
    @abstractmethod
    def water_heater_temperature_step(self) -> int:
        """Abstract method for get water heater temperature step"""
        raise NotImplementedError

    @abstractmethod
    def set_water_heater_operation_mode(self, operation_mode: str) -> None:
        """Abstract method for set water heater operation mode"""
        raise NotImplementedError

    @abstractmethod
    async def async_set_water_heater_operation_mode(self, operation_mode: str) -> None:
        """Abstract method for async set water heater operation mode"""
        raise NotImplementedError

    @property
    def central_heating_total_energy_consumption(self) -> Any:
        """Get central heating total energy consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.CENTRAL_HEATING_TOTAL_ENERGY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def domestic_hot_water_total_energy_consumption(self) -> Any:
        """Get domestic hot water total energy consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_TOTAL_ENERGY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def central_heating_gas_consumption(self) -> Any:
        """Get central heating gas consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.CENTRAL_HEATING_GAS,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def domestic_hot_water_heating_pump_electricity_consumption(self) -> Any:
        """Get domestic hot water heating pump electricity consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_HEATING_PUMP_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def domestic_hot_water_resistor_electricity_consumption(self) -> Any:
        """Get domestic hot water resistor electricity consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_RESISTOR_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def domestic_hot_water_gas_consumption(self) -> Any:
        """Get domestic hot water gas consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_GAS,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def central_heating_electricity_consumption(self) -> Any:
        """Get central heating electricity consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.CENTRAL_HEATING_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    @property
    def domestic_hot_water_electricity_consumption(self) -> Any:
        """Get domestic hot water electricity consumption"""
        return self._get_consumption_sequence_last_value(
            ConsumptionType.DOMESTIC_HOT_WATER_ELECTRICITY,
            ConsumptionTimeInterval.LAST_DAY,
        )

    def _get_consumption_sequence_last_value(
        self,
        consumption_type: ConsumptionType,
        time_interval: ConsumptionTimeInterval,
    ) -> Any:
        """Get last value for consumption sequence"""
        for sequence in self.consumptions_sequences:
            if sequence["k"] == consumption_type.value and sequence["p"] == time_interval.value:
                return sequence["v"][-1]

        return None

    def _update_energy(self, old_consumptions_sequences: Optional[list[dict[str, Any]]]) -> None:
        """Update the device energy settings"""
        if (
            self.custom_features.get(
                ConsumptionType.DOMESTIC_HOT_WATER_ELECTRICITY.name
            )
            is None
        ):
            self._set_energy_features()

        if (
            old_consumptions_sequences is not None
            and len(old_consumptions_sequences) > 0
            and old_consumptions_sequences != self.consumptions_sequences
        ):
            self.consumption_sequence_last_changed_utc = dt.datetime.now(
                dt.timezone.utc
            ) - dt.timedelta(hours=1)

    def update_energy(self) -> None:
        """Update the device energy settings from the cloud"""
        old_consumptions_sequences = self.consumptions_sequences
        self._get_consumptions_sequences()
        self._update_energy(old_consumptions_sequences)

    async def async_update_energy(self) -> None:
        """Async update the device energy settings from the cloud"""
        old_consumptions_sequences = self.consumptions_sequences
        await self._async_get_consumptions_sequences()
        self._update_energy(old_consumptions_sequences)

    def get_bus_errors(self) -> None:
        """Get bus errors from the cloud"""
        self.bus_errors_list = self.api.get_bus_errors(self.gw)

    async def async_get_bus_errors(self) -> None:
        """Async get bus errors from the cloud"""
        self.bus_errors_list = await self.api.async_get_bus_errors(self.gw)

    def _set_energy_features(self):
        """Set energy features"""
        for consumption_type in ConsumptionType:
            if (
                self._get_consumption_sequence_last_value(
                    consumption_type,
                    ConsumptionTimeInterval.LAST_DAY,
                )
                is not None
            ):
                self.custom_features[consumption_type.name] = True
            else:
                self.custom_features[consumption_type.name] = False

    def are_device_features_available(
        self,
        device_features: Optional[list[DeviceFeatures]],
        system_types: Optional[list[SystemType]],
        whe_types: Optional[list[WheType]],
    ) -> bool:
        """Checks features availability"""
        if system_types is not None and self.system_type not in system_types:
            return False

        if whe_types is not None and self.whe_type not in whe_types:
            return False

        if device_features is not None:
            for device_feature in device_features:
                if (
                    self.features.get(str(device_feature)) is not True
                    and self.custom_features.get(str(device_feature)) is not True
                ):
                    return False

        return True



"""Constants for ariston module"""
from enum import Enum, IntEnum, unique
from typing import Final

ARISTON_API_URL: Final[str] = "https://www.ariston-net.remotethermo.com/api/v2/"
ARISTON_LOGIN: Final[str] = "accounts/login"
ARISTON_REMOTE: Final[str] = "remote"
ARISTON_VELIS: Final[str] = "velis"
ARISTON_PLANTS: Final[str] = "plants"
ARISTON_LITE: Final[str] = "lite"
ARISTON_DATA_ITEMS: Final[str] = "dataItems"
ARISTON_ZONES: Final[str] = "zones"
ARISTON_BSB_ZONES: Final[str] = "bsbZones"
ARISTON_PLANT_DATA: Final[str] = "plantData"
ARISTON_REPORTS: Final[str] = "reports"
ARISTON_TIME_PROGS: Final[str] = "timeProgs"
ARISTON_BUS_ERRORS: Final[str] = "busErrors"


@unique
class PlantData(str, Enum):
    """Plant data enum"""

    Med = "medPlantData"
    Se = "sePlantData"
    Slp = "slpPlantData"
    Bsb = "bsbPlantData"


@unique
class PlantMode(Enum):
    """Plant mode enum"""

    UNDEFINED = -1
    SUMMER = 0
    WINTER = 1
    HEATING_ONLY = 2
    COOLING = 3
    COOLING_ONLY = 4
    OFF = 5
    HOLIDAY = 6


@unique
class ZoneMode(Enum):
    """Zone mode enum"""

    UNDEFINED = -1
    OFF = 0
    MANUAL_NIGHT = 1
    MANUAL = 2
    TIME_PROGRAM = 3

@unique
class BsbZoneMode(Enum):
    """BSB zone mode enum"""

    UNDEFINED = -1
    OFF = 0
    TIME_PROGRAM = 1
    MANUAL = 2
    MANUAL_NIGHT = 3

@unique
class DhwMode(Enum):
    """Dhw mode enum"""

    DISABLED = 0
    TIME_BASED = 1
    ALWAYS_ACTIVE = 2
    HC_HP = 3
    HC_HP_40 = 4
    GREEN = 5


@unique
class Weather(Enum):
    """Weather enum"""

    UNAVAILABLE = 0
    CLEAR = 1
    VARIABLE = 2
    CLOUDY = 3
    RAINY = 4
    RAINSTORM = 5
    SNOW = 6
    FOG = 7
    WINDY = 8
    CLEAR_BY_NIGHT = 129
    VARIABLE_BY_NIGHT = 130


@unique
class GasEnergyUnit(IntEnum):
    """Gas energy unit enum"""

    KWH = 0
    GIGA_JOULE = 1
    THERM = 2
    MEGA_BTU = 3
    SMC = 4
    CUBE_METER = 5


@unique
class GasType(IntEnum):
    """Gas type enu,"""

    NATURAL_GAS = 0
    LPG = 1
    AIR_PROPANED = 2
    GPO = 3
    PROPANE = 4


@unique
class Currency(IntEnum):
    """Currency enum"""

    ARS = 1
    EUR = 2
    BYN = 3
    CNY = 4
    HRK = 5
    CZK = 6
    DKK = 7
    HKD = 8
    HUF = 9
    IRR = 10
    KZT = 11
    CHF = 12
    MOP = 13
    PLZ = 14
    RON = 15
    RUB = 16
    TRY = 17
    UAH = 18
    GBP = 19
    USD = 20


@unique
class SystemType(Enum):
    """System type enum"""

    UNKNOWN = -1
    GALILEO1 = 1
    GALILEO2 = 2
    GALEVO = 3
    VELIS = 4
    BSB = 5


@unique
class Brands(Enum):
    """Brands enum"""

    Ariston = 1
    Chaffoteaux = 2
    Elco = 3
    Atag = 4
    Nti = 5
    Htp = 6
    Racold = 7


class WaterHeaterMode(Enum):
    """Base class for plant modes"""


@unique
class BsbOperativeMode(WaterHeaterMode):
    """BSB operative mode enum"""

    OFF = 0
    ON = 1

@unique
class LuxPlantMode(WaterHeaterMode):
    """Lux plant mode enum"""

    MANUAL = 1
    PROGRAM = 5
    BOOST = 9


@unique
class EvoPlantMode(WaterHeaterMode):
    """Evo plant mode enum"""

    MANUAL = 1
    PROGRAM = 5


@unique
class VelisPlantMode(Enum):
    """Velis plant mode enum"""

    MANUAL = 1
    PROGRAM = 5
    NIGHT = 8


@unique
class NuosSplitPlantMode(Enum):
    """NuosSplit plant mode enum"""

    MANUAL = 1
    PROGRAM = 2


@unique
class NuosSplitOperativeMode(WaterHeaterMode):
    """NuosSplit operative mode enum"""

    GREEN = 0
    COMFORT = 1
    FAST = 2
    IMEMORY = 3


@unique
class LydosPlantMode(WaterHeaterMode):
    """Lydos hybrid plant mode enum"""

    IMEMORY = 1
    GREEN = 2
    PROGRAM = 6
    BOOST = 7


@unique
class WheType(Enum):
    """Whe type enum"""

    Unknown = -1
    Evo = 1
    LydosHybrid = 2
    NuosSplit = 4
    Andris2 = 5
    Evo2 = 6
    Lux2 = 7
    Lux = 8


@unique
class ConsumptionType(Enum):
    """Consumption type"""

    CENTRAL_HEATING_TOTAL_ENERGY = 1
    DOMESTIC_HOT_WATER_TOTAL_ENERGY = 2
    CENTRAL_HEATING_GAS = 7
    DOMESTIC_HOT_WATER_HEATING_PUMP_ELECTRICITY = 8
    DOMESTIC_HOT_WATER_RESISTOR_ELECTRICITY = 9
    DOMESTIC_HOT_WATER_GAS = 10
    CENTRAL_HEATING_ELECTRICITY = 20
    DOMESTIC_HOT_WATER_ELECTRICITY = 21


@unique
class ConsumptionTimeInterval(Enum):
    """Consumption time interval"""

    # I am not sure. This is just a guess.

    LAST_DAY = 1
    LAST_WEEK = 2
    LAST_MONTH = 3
    LAST_YEAR = 4


class DeviceAttribute:
    """Constants for device attributes"""

    GW: Final[str] = "gw"  # gwId
    HPMP_SYS: Final[str] = "hpmpSys"
    IS_OFFLINE_48H: Final[str] = "isOffline48H"
    LNK: Final[str] = "lnk"  # gwLink
    LOC: Final[str] = "loc"  # location
    CONSUMPTION_SETTINGS: Final[str] = "consumptionsSettings"
    GEOFENCE_CONFIG: Final[str] = "geofenceConfig"
    MQTT_API_VERSION: Final[str] = "mqttApiVersion"  # mqttApiVersion
    NAME: Final[str] = "name"  # plantName
    SN: Final[str] = "sn"  # gwSerial
    SYS: Final[str] = "sys"  # gwSysType
    TC_BY_GUEST: Final[str] = "tcByGuest"  # controlledByGuest
    UTC_OFT: Final[str] = "utcOft"
    WEATHER_PROVIDER: Final[str] = "weatherProvider"


class GalevoDeviceAttribute(DeviceAttribute):
    """Constants for galevo device attributes"""

    ZONES: Final[str] = "zones"
    SOLAR: Final[str] = "solar"
    CONV_BOILER: Final[str] = "convBoiler"
    HYBRID_SYS: Final[str] = "hybridSys"
    DHW_PROG_SUPPORTED: Final[str] = "dhwProgSupported"
    VIRTUAL_ZONES: Final[str] = "virtualZones"
    HAS_VMC: Final[str] = "hasVmc"
    HAS_EXT_TP: Final[str] = "hasExtTP"  # extendedTimeProg
    HAS_BOILER: Final[str] = "hasBoiler"
    PILOT_SUPPORTED: Final[str] = "pilotSupported"
    UMSYS: Final[str] = "umsys"
    IS_VMC_R2: Final[str] = "isVmcR2"
    IS_EVO2: Final[str] = "isEvo2"
    FW_VER: Final[str] = "fwVer"


class VelisDeviceAttribute(DeviceAttribute):
    """Constants for velis device attributes"""

    NOTIFY_ON_CONDENSATE_TANK_FULL: Final[str] = "notifyOnCondensateTankFull"
    NOTIFY_ON_ERRORS: Final[str] = "notifyOnErrors"
    NOTIFY_ON_READY_SHOWERS: Final[str] = "notifyOnReadyShowers"
    WHE_MODEL_TYPE: Final[str] = "wheModelType"
    WHE_TYPE: Final[str] = "wheType"


class ZoneAttribute:
    """Constants for zone attributes"""

    NUM: Final[str] = "num"
    NAME: Final[str] = "name"
    ROOM_SENS: Final[str] = "roomSens"
    GEOFENCE_DEROGA: Final[str] = "geofenceDeroga"
    IS_HIDDEN: Final[str] = "isHidden"


class CustomDeviceFeatures:
    """Constants for custom device features"""

    HAS_DHW: Final[str] = "hasDhw"
    HAS_OUTSIDE_TEMP: Final[str] = "hasOutsideTemp"


class DeviceFeatures:
    """Constants for device features"""

    AUTO_THERMO_REG: Final[str] = "autoThermoReg"
    BMS_ACTIVE: Final[str] = "bmsActive"
    BUFFER_TIME_PROG_AVAILABLE: Final[str] = "bufferTimeProgAvailable"
    CASCDE_SYS: Final[str] = "cascadeSys"
    CONV_BOILER: Final[str] = "convBoiler"
    DHW_BOILER_PRESENT: Final[str] = "dhwBoilerPresent"
    DHW_HIDDEN: Final[str] = "dhwHidden"
    DHW_MODE_CHANGEABLE: Final[str] = "dhwModeChangeable"
    DHW_PROG_SUPPORTED: Final[str] = "dhwProgSupported"
    DICTINCT_HEAT_COOL_SETPOINT: Final[str] = "distinctHeatCoolSetpoints"
    EXTENDED_TIME_PROG: Final[str] = "extendedTimeProg"
    HAS_BOILER: Final[str] = "hasBoiler"
    HAS_EM20: Final[str] = "hasEm20"
    HAS_FIREPLACE: Final[str] = "hasFireplace"
    HAS_METERING: Final[str] = "hasMetering"
    HAS_SLP: Final[str] = "hasSlp"  # Low Pressure Pump
    HAS_TWO_COOLING_TEMP: Final[str] = "hasTwoCoolingTemp"
    HAS_VMC: Final[str] = "hasVmc"
    HAS_ZONE_NAMES: Final[str] = "hasZoneNames"
    HP_CASCADE_CONFIG: Final[str] = "hpCascadeConfig"
    HP_CASCADE_SYS: Final[str] = "hpCascadeSys"
    HP_SYS: Final[str] = "hpSys"
    HV_INPUT_OFF: Final[str] = "hvInputOff"
    HYBRID_SYS: Final[str] = "hybridSys"
    IS_EVO2: Final[str] = "isEvo2"
    IS_VMC_R2: Final[str] = "isVmcR2"
    PILOT_SUPPORTED: Final[str] = "pilotSupported"
    PRE_HEATING_SUPPORTED: Final[str] = "preHeatingSupported"
    SOLAR: Final[str] = "solar"
    VIRTUAL_ZONES: Final[str] = "virtualZones"
    ZONES: Final[str] = "zones"
    WEATHER_PROVIDER: Final[str] = "weatherProvider"


class VelisDeviceProperties:
    """Contants for Velis device properties"""

    GW: Final[str] = "gw"
    MODE: Final[str] = "mode"
    ON: Final[str] = "on"
    PROC_REQ_TEMP: Final[str] = "procReqTemp"


class NuosSplitProperties(VelisDeviceProperties):
    """Constants for NuosSplit device properties"""

    WATER_TEMP: Final[str] = "waterTemp"
    COMFORT_TEMP: Final[str] = "comfortTemp"
    REDUCED_TEMP: Final[str] = "reducedTemp"
    OP_MODE: Final[str] = "opMode"
    BOOST_ON: Final[str] = "boostOn"
    HP_STATE: Final[str] = "hpState"


class EvoLydosDeviceProperties(VelisDeviceProperties):
    """Constants for evo and lydos device properties"""

    ANTI_LEG: Final[str] = "antiLeg"
    AV_SHW: Final[str] = "avShw"
    HEAT_REQ: Final[str] = "heatReq"
    REQ_TEMP: Final[str] = "reqTemp"
    TEMP: Final[str] = "temp"


class EvoDeviceProperties(EvoLydosDeviceProperties):
    """Contants for Velis Evo device properties"""

    ECO: Final[str] = "eco"
    PWR_OPT: Final[str] = "pwrOpt"
    RM_TM: Final[str] = "rmTm"


class LydosDeviceProperties(EvoLydosDeviceProperties):
    """Contants for Velis Lydos device properties"""

    BOOST_REQ_TEMP: Final[str] = "boostReqTemp"

class BsbDeviceProperties:
    """Constants for bsb device properties."""

    DHW_COMF_TEMP: Final[str] = "dhwComfTemp"
    DHW_ENABLED: Final[str] = "dhwEnabled"
    DHW_MODE: Final[str] = "dhwMode"
    DHW_PROG_READ_ONLY: Final[str] = "dhwProgReadOnly"
    DHW_REDU_TEMP: Final[str] = "dhwReduTemp"
    DHW_STORAGE_TEMP_ERROR: Final[str] = "dhwStorageTempError"
    DHW_TEMP: Final[str] = "dhwTemp"
    FLAME: Final[str] = "flame"
    GW: Final[str] = "gw"
    HAS_DHW_TEMP: Final[str] = "hasDhwTemp"
    HAS_OUT_TEMP: Final[str] = "hasOutTemp"
    HP_ON: Final[str] = "hpOn"
    OUTSIDE_TEMP_ERROR: Final[str] = "outsideTempError"
    OUT_TEMP: Final[str] = "outTemp"
    ZONES: Final[str] = "zones"


class BsbZoneProperties:
    """Constants for bsb zone properties."""

    CH_COMF_TEMP: Final[str] = "chComfTemp"
    CH_PROT_TEMP: Final[str] = "chProtTemp"
    CH_RED_TEMP: Final[str] = "chRedTemp"
    COOL_COMF_TEMP: Final[str] = "coolComfTemp"
    COOLING_ON: Final[str] = "coolingOn"
    COOL_PROT_TEMP: Final[str] = "coolProtTemp"
    COOL_RED_TEMP: Final[str] = "coolRedTemp"
    DESIRED_ROOM_TEMP: Final[str] = "desiredRoomTemp"
    HAS_ROOM_SENS: Final[str] = "hasRoomSens"
    HEATING_ON: Final[str] = "heatingOn"
    HEAT_OR_COOL_REQ: Final[str] = "heatOrCoolReq"
    HOLIDAYS: Final[str] = "holidays"
    MODE: Final[str] = "mode"
    ROOM_TEMP: Final[str] = "roomTemp"
    ROOM_TEMP_ERROR: Final[str] = "roomTempError"
    USE_REDUCED_OPERATION_MODE_ON_HOLIDAY: Final[str] = "useReducedOperationModeOnHoliday"

class MedDeviceSettings:
    """Constatns for Med device settings"""

    MED_ANTILEGIONELLA_ON_OFF: Final[str] = "MedAntilegionellaOnOff"
    MED_HEATING_RATE: Final[str] = "MedHeatingRate"
    MED_MAX_SETPOINT_TEMPERATURE: Final[str] = "MedMaxSetpointTemperature"
    MED_MAX_SETPOINT_TEMPERATURE_MAX: Final[str] = "MedMaxSetpointTemperatureMax"
    MED_MAX_SETPOINT_TEMPERATURE_MIN: Final[str] = "MedMaxSetpointTemperatureMin"


class SeDeviceSettings:
    """Constatns for Se device settings"""

    SE_ANTILEGIONELLA_ON_OFF: Final[str] = "SeAntilegionellaOnOff"
    SE_ANTI_COOLING_ON_OFF: Final[str] = "SeAntiCoolingOnOff"
    SE_NIGHT_MODE_ON_OFF: Final[str] = "SeNightModeOnOff"
    SE_PERMANENT_BOOST_ON_OFF: Final[str] = "SePermanentBoostOnOff"
    SE_MAX_SETPOINT_TEMPERATURE: Final[str] = "SeMaxSetpointTemperature"
    SE_MAX_SETPOINT_TEMPERATURE_MAX: Final[str] = "SeMaxSetpointTemperatureMax"
    SE_MAX_SETPOINT_TEMPERATURE_MIN: Final[str] = "SeMaxSetpointTemperatureMin"
    SE_ANTI_COOLING_TEMPERATURE: Final[str] = "SeAntiCoolingTemperature"
    SE_ANTI_COOLING_TEMPERATURE_MAX: Final[str] = "SeAntiCoolingTemperatureMin"
    SE_ANTI_COOLING_TEMPERATURE_MIN: Final[str] = "SeAntiCoolingTemperatureMax"
    SE_MAX_GREEN_SETPOINT_TEMPERATURE: Final[str] = "SeMaxGreenSetpointTemperature"
    SE_HEATING_RATE: Final[str] = "SeHeatingRate"
    SE_NIGHT_BEGIN_AS_MINUTES: Final[str] = "SeNightBeginAsMinutes"
    SE_NIGHT_BEGIN_MIN_AS_MINUTES: Final[str] = "SeNightBeginMinAsMinutes"
    SE_NIGHT_BEGIN_MAX_AS_MINUTES: Final[str] = "SeNightBeginMaxAsMinutes"
    SE_NIGHT_END_AS_MINUTES: Final[str] = "SeNightEndAsMinutes"
    SE_NIGHT_END_MIN_AS_MINUTES: Final[str] = "SeNightEndMinAsMinutes"
    SE_NIGHT_END_MAX_AS_MINUTES: Final[str] = "SeNightEndMaxAsMinutes"


class SlpDeviceSettings:
    """Constatns for Slp device settings"""

    SLP_MAX_GREEN_TEMPERATURE: Final[str] = "SlpMaxGreenTemperature"
    SLP_MAX_SETPOINT_TEMPERATURE: Final[str] = "SlpMaxSetpointTemperature"
    SLP_MAX_SETPOINT_TEMPERATURE_MIN: Final[str] = "SlpMaxSetpointTemperatureMin"
    SLP_MAX_SETPOINT_TEMPERATURE_MAX: Final[str] = "SlpMaxSetpointTemperatureMax"
    SLP_MIN_SETPOINT_TEMPERATURE: Final[str] = "SlpMinSetpointTemperature"
    SLP_MIN_SETPOINT_TEMPERATURE_MIN: Final[str] = "SlpMinSetpointTemperatureMin"
    SLP_MIN_SETPOINT_TEMPERATURE_MAX: Final[str] = "SlpMinSetpointTemperatureMax"
    SLP_ANTILEGIONELLA_ON_OFF: Final[str] = "SlpAntilegionellaOnOff"
    SLP_PRE_HEATING_ON_OFF: Final[str] = "SlpPreHeatingOnOff"
    SLP_HEATING_RATE: Final[str] = "SlpHeatingRate"
    SLP_HC_HP_MODE: Final[str] = "SlpHcHpMode"


class DeviceProperties:
    """Constants for device properties"""

    PLANT_MODE: Final[str] = "PlantMode"
    IS_FLAME_ON: Final[str] = "IsFlameOn"
    IS_HEATING_PUMP_ON: Final[str] = "IsHeatingPumpOn"
    HOLIDAY: Final[str] = "Holiday"
    OUTSIDE_TEMP: Final[str] = "OutsideTemp"
    WEATHER: Final[str] = "Weather"
    HEATING_CIRCUIT_PRESSURE: Final[str] = "HeatingCircuitPressure"
    CH_FLOW_TEMP: Final[str] = "ChFlowTemp"
    CH_FLOW_SETPOINT_TEMP: Final[str] = "ChFlowSetpointTemp"
    DHW_TEMP: Final[str] = "DhwTemp"
    DHW_STORAGE_TEMPERATURE: Final[str] = "DhwStorageTemperature"
    DHW_TIMEPROG_COMFORT_TEMP: Final[str] = "DhwTimeProgComfortTemp"
    DHW_TIMEPROG_ECONOMY_TEMP: Final[str] = "DhwTimeProgEconomyTemp"
    DHW_MODE: Final[str] = "DhwMode"
    AUTOMATIC_THERMOREGULATION: Final[str] = "AutomaticThermoregulation"
    ANTILEGIONELLA_ON_OFF: Final[str] = "AntilegionellaOnOff"
    ANTILEGIONELLA_TEMP: Final[str] = "AntilegionellaTemp"
    ANTILEGIONELLA_FREQ: Final[str] = "AntilegionellaFreq"
    HYBRID_MODE: Final[str] = "HybridMode"
    BUFFER_CONTROL_MODE: Final[str] = "BufferControlMode"
    BUFFER_TIME_PROG_COMFORT_HEATING_TEMP: Final[str] = "BufferTimeProgComfortHeatingTemp"
    BUFFER_TIME_PROG_ECONOMY_HEATING_TEMP: Final[str] = "BufferTimeProgEconomyHeatingTemp"
    BUFFER_TIME_PROG_COMFORT_COOLING_TEMP: Final[str] = "BufferTimeProgComfortCoolingTemp"
    BUFFER_TIME_PROG_ECONOMY_COOLING_TEMP: Final[str] = "BufferTimeProgEconomyCoolingTemp"
    IS_QUITE: Final[str] = "IsQuite"


class ThermostatProperties:
    """Constants for thermostat properties"""

    ZONE_MEASURED_TEMP: Final[str] = "ZoneMeasuredTemp"
    ZONE_DESIRED_TEMP: Final[str] = "ZoneDesiredTemp"
    ZONE_COMFORT_TEMP: Final[str] = "ZoneComfortTemp"
    ZONE_MODE: Final[str] = "ZoneMode"
    ZONE_HEAT_REQUEST: Final[str] = "ZoneHeatRequest"
    ZONE_ECONOMY_TEMP: Final[str] = "ZoneEconomyTemp"
    ZONE_DEROGA: Final[str] = "ZoneDeroga"
    ZONE_IS_ZONE_PILOT_ON: Final[str] = "IsZonePilotOn"
    ZONE_VIRT_TEMP_OFFSET_HEAT: Final[str] = "VirtTempOffsetHeat"
    HEATING_FLOW_TEMP: Final[str] = "HeatingFlowTemp"
    HEATING_FLOW_OFFSET: Final[str] = "HeatingFlowOffset"
    COOLING_FLOW_TEMP: Final[str] = "CoolingFlowTemp"
    COOLING_FLOW_OFFSET: Final[str] = "CoolingFlowOffset"
    ZONE_NAME: Final[str] = "ZoneName"
    VIRT_TEMP_SETPOINT_HEAT: Final[str] = "VirtTempSetpointHeat"
    VIRT_TEMP_SETPOINT_COOL: Final[str] = "VirtTempSetpointCool"
    VIRT_COMFORT_TEMP: Final[str] = "VirtComfortTemp"
    VIRT_REDUCED_TEMP: Final[str] = "VirtReducedTemp"
    ZONE_VIRT_TEMP_OFFSET_COOL: Final[str] = "VirtTempOffsetCool"


class ConsumptionProperties:
    """Constants for consumption properties"""

    CURRENCY: Final[str] = "currency"
    GAS_TYPE: Final[str] = "gasType"
    GAS_ENERGY_UNIT: Final[str] = "gasEnergyUnit"
    ELEC_COST: Final[str] = "elecCost"
    GAS_COST: Final[str] = "gasCost"


class PropertyType:
    """Constants for property types"""

    VALUE: Final[str] = "value"
    OPTIONS: Final[str] = "options"
    OPT_TEXTS: Final[str] = "optTexts"
    UNIT: Final[str] = "unit"
    MIN: Final[str] = "min"
    MAX: Final[str] = "max"
    STEP: Final[str] = "step"
    DECIMALS: Final[str] = "decimals"
    ZONE: Final[str] = "zone"
    EXPIRES_ON: Final[str] = "expiresOn"
    ALLOWED_OPTIONS: Final[str] = "allowedOptions"

class BusErrorsProperties:
    """Constants for bus errors properties"""
    GW: Final[str] = "gw"
    TIMESTAMP: Final[str] = "timestamp"
    FAULT: Final[str] = "fault"
    MULT: Final[str] = "mult"
    CODE: Final[str] = "code"
    PRI: Final[str] = "pri"
    ERR_DEX: Final[str] = "errDex"
    RES: Final[str] = "res"
    BLK: Final[str] = "blk"



"""BSB device class for Ariston module."""
import logging
from typing import Any, Optional

from .const import (
    BsbDeviceProperties,
    BsbOperativeMode,
    BsbZoneMode,
    BsbZoneProperties,
    CustomDeviceFeatures,
    PropertyType,
)
from .device import AristonDevice

_LOGGER = logging.getLogger(__name__)


class AristonBsbDevice(AristonDevice):
    """Class representing a physical device, it's state and properties."""

    @property
    def consumption_type(self) -> str:
        """String to get consumption type"""
        return f"Ch{'%2CDhw'}"

    @property
    def plant_mode_supported(self) -> bool:
        """Returns is plant mode supported"""
        return False

    def update_state(self) -> None:
        """Update the device states from the cloud."""
        self.data = self.api.get_bsb_plant_data(self.gw)

    async def async_update_state(self) -> None:
        """Async update the device states from the cloud."""
        self.data = await self.api.async_get_bsb_plant_data(self.gw)

    def _get_features(self) -> None:
        self.custom_features[CustomDeviceFeatures.HAS_DHW] = True
        self.custom_features[CustomDeviceFeatures.HAS_OUTSIDE_TEMP] = True

    def get_features(self) -> None:
        """Get device features wrapper"""
        super().get_features()
        self._get_features()

    async def async_get_features(self) -> None:
        """Async get device features wrapper"""
        await super().async_get_features()
        self._get_features()

    @property
    def zone_numbers(self) -> list[int]:
        """Get zone number for device"""
        return [int(zone) for zone in self.zones]

    @property
    def zones(self) -> dict[str, dict[str, Any]]:
        """Get device zones wrapper"""
        return self.data.get(BsbDeviceProperties.ZONES, dict[str, dict[str, Any]]())

    def get_zone(self, zone: int) -> dict[str, Any]:
        """Get device zone"""
        return self.zones.get(str(zone), dict())

    def get_zone_ch_comf_temp(self, zone: int) -> dict[str, Any]:
        """Get device zone central heating comfort temperature"""
        return self.get_zone(zone).get(BsbZoneProperties.CH_COMF_TEMP, dict[str, Any]())

    def get_zone_ch_red_temp(self, zone: int) -> dict[str, Any]:
        """Get device zone central heating reduced temperature"""
        return self.get_zone(zone).get(BsbZoneProperties.CH_RED_TEMP, dict[str, Any]())

    def get_zone_mode(self, zone: int) -> BsbZoneMode:
        """Get zone mode on value"""
        zone_mode = (
            self.get_zone(zone)
            .get(BsbZoneProperties.MODE, dict[str, Any]())
            .get(PropertyType.VALUE, None)
        )

        if zone_mode is None:
            return BsbZoneMode.UNDEFINED

        return BsbZoneMode(zone_mode)

    def get_zone_mode_options(self, zone: int) -> list[int]:
        """Get zone mode on options"""
        return (
            self.get_zone(zone)
            .get(BsbZoneProperties.MODE, dict[str, Any]())
            .get(PropertyType.ALLOWED_OPTIONS, None)
        )

    @property
    def is_plant_in_heat_mode(self) -> bool:
        """Is the plant in a heat mode"""
        return not self.is_plant_in_cool_mode

    @property
    def is_plant_in_cool_mode(self) -> bool:
        """Is the plant in a cool mode"""
        return self.get_zone(self.zone_numbers[0]).get(
            BsbZoneProperties.COOLING_ON, False
        )

    def is_zone_in_manual_mode(self, zone: int) -> bool:
        """Is zone in manual mode"""
        return self.get_zone_mode(zone) in [
            BsbZoneMode.MANUAL,
            BsbZoneMode.MANUAL_NIGHT,
        ]

    def is_zone_in_time_program_mode(self, zone: int) -> bool:
        """Is zone in time program mode"""
        return self.get_zone_mode(zone) in [
            BsbZoneMode.TIME_PROGRAM,
        ]

    def is_zone_mode_options_contains_manual(self, zone: int) -> bool:
        """Is zone mode options contains manual mode"""
        return (
            BsbZoneMode.MANUAL.value or BsbZoneMode.MANUAL_NIGHT.value
        ) in self.get_zone_mode_options(zone)

    def is_zone_mode_options_contains_time_program(self, zone: int) -> bool:
        """Is zone mode options contains time program mode"""
        return BsbZoneMode.TIME_PROGRAM.value in self.get_zone_mode_options(zone)

    def is_zone_mode_options_contains_off(self, zone: int) -> bool:
        """Is zone mode options contains off mode"""
        return BsbZoneMode.OFF.value in self.get_zone_mode_options(zone)

    @property
    def is_flame_on_value(self) -> bool:
        """Get is flame on value"""
        return self.data.get(BsbDeviceProperties.FLAME, False)

    @property
    def water_heater_current_temperature(self) -> Optional[float]:
        """Method for getting current water heater temperature."""
        return self.data.get(BsbDeviceProperties.DHW_TEMP, None)

    @property
    def water_heater_minimum_temperature(self) -> float:
        """Method for getting water heater minimum temperature"""
        return self.data.get(BsbDeviceProperties.DHW_COMF_TEMP, dict[str, Any]()).get(
            PropertyType.MIN, None
        )

    @property
    def water_heater_reduced_minimum_temperature(self) -> Optional[float]:
        """Get water heater reduced temperature"""
        return self.data.get(BsbDeviceProperties.DHW_REDU_TEMP, dict[str, Any]()).get(
            PropertyType.MIN, None
        )

    @property
    def water_heater_target_temperature(self) -> Optional[float]:
        """Method for getting water heater target temperature"""
        return self.data.get(BsbDeviceProperties.DHW_COMF_TEMP, dict[str, Any]()).get(
            PropertyType.VALUE, None
        )

    @property
    def water_heater_reduced_temperature(self) -> Optional[float]:
        """Get water heater reduced temperature"""
        return self.data.get(BsbDeviceProperties.DHW_REDU_TEMP, dict[str, Any]()).get(
            PropertyType.VALUE, None
        )

    @property
    def water_heater_maximum_temperature(self) -> Optional[float]:
        """Method for getting water heater maximum temperature"""
        return self.data.get(BsbDeviceProperties.DHW_COMF_TEMP, dict[str, Any]()).get(
            PropertyType.MAX, None
        )

    @property
    def water_heater_reduced_maximum_temperature(self) -> Optional[float]:
        """Get water heater reduced temperature"""
        return self.data.get(BsbDeviceProperties.DHW_REDU_TEMP, dict[str, Any]()).get(
            PropertyType.MAX, None
        )

    @property
    def water_heater_temperature_step(self) -> int:
        """Method for getting water heater temperature step"""
        return self.data.get(BsbDeviceProperties.DHW_COMF_TEMP, dict[str, Any]()).get(
            PropertyType.STEP, None
        )

    @property
    def water_heater_reduced_temperature_step(self) -> Optional[float]:
        """Get water heater reduced temperature"""
        return self.data.get(BsbDeviceProperties.DHW_REDU_TEMP, dict[str, Any]()).get(
            PropertyType.STEP, None
        )

    @property
    def water_heater_temperature_decimals(self) -> int:
        """Method for getting water heater temperature decimals"""
        return 1

    @property
    def water_heater_temperature_unit(self) -> str:
        """Get water heater temperature unit"""
        return "°C"

    @property
    def water_heater_mode_operation_texts(self) -> list[str]:
        """Get water heater operation mode texts"""
        return [flag.name for flag in BsbOperativeMode]

    @property
    def water_heater_mode_options(self) -> list[int]:
        """Get water heater operation options"""
        return [flag.value for flag in BsbOperativeMode]

    @property
    def water_heater_mode_value(self) -> Optional[int]:
        """Method for getting water heater mode value"""
        return self.data.get(BsbDeviceProperties.DHW_MODE, dict[str, Any]()).get(
            PropertyType.VALUE, None
        )

    def get_comfort_temp_min(self, zone: int) -> int:
        """Get zone comfort temp min"""
        return self.get_zone_ch_comf_temp(zone).get(PropertyType.MIN, 15)

    def get_comfort_temp_max(self, zone: int) -> int:
        """Get zone comfort temp max"""
        return self.get_zone_ch_comf_temp(zone).get(PropertyType.MAX, 24)

    def get_comfort_temp_step(self, zone: int) -> int:
        """Get zone comfort temp step"""
        return self.get_zone_ch_comf_temp(zone).get(PropertyType.STEP, 0.5)

    def get_comfort_temp_value(self, zone: int) -> int:
        """Get zone comfort temp value"""
        return self.get_zone_ch_comf_temp(zone).get(PropertyType.VALUE, 0)

    def get_reduced_temp_min(self, zone: int) -> int:
        """Get zone reduced temp min"""
        return self.get_zone_ch_red_temp(zone).get(PropertyType.MIN, 10)

    def get_reduced_temp_max(self, zone: int) -> int:
        """Get zone reduced temp max"""
        return self.get_zone_ch_red_temp(zone).get(PropertyType.MAX, 18)

    def get_reduced_temp_step(self, zone: int) -> int:
        """Get zone reduced temp step"""
        return self.get_zone_ch_red_temp(zone).get(PropertyType.STEP, 0.5)

    def get_reduced_temp_value(self, zone: int) -> int:
        """Get zone reduced temp value"""
        return self.get_zone_ch_red_temp(zone).get(PropertyType.VALUE, 0)

    def get_measured_temp_value(self, zone: int) -> int:
        """Get zone measured temp value"""
        return self.get_zone(zone).get(BsbZoneProperties.ROOM_TEMP, 0)

    def get_measured_temp_decimals(self, zone: int) -> int:
        """Get zone measured temp decimals"""
        return 1

    def get_measured_temp_unit(self, zone: int) -> str:
        """Get zone measured temp unit"""
        return "°C"

    def set_water_heater_temperature(self, temperature: float):
        """Set water heater temperature"""
        if len(self.data) == 0:
            self.update_state()
        reduced = self.water_heater_reduced_temperature
        if reduced is None:
            reduced = 0
        self._set_water_heater_temperature(temperature, reduced)

    async def async_set_water_heater_temperature(self, temperature: float):
        """Async set water heater temperature"""
        if len(self.data) == 0:
            await self.async_update_state()
        reduced = self.water_heater_reduced_temperature
        if reduced is None:
            reduced = 0
        await self._async_set_water_heater_temperature(temperature, reduced)

    def set_water_heater_operation_mode(self, operation_mode: str) -> None:
        """Set water heater operation mode"""
        self.api.set_bsb_mode(self.gw, BsbOperativeMode[operation_mode])
        self.data[BsbDeviceProperties.DHW_MODE][PropertyType.VALUE] = BsbOperativeMode[operation_mode].value

    async def async_set_water_heater_operation_mode(self, operation_mode: str) -> None:
        """Async set water heater operation mode"""
        await self.api.async_set_bsb_mode(self.gw, BsbOperativeMode[operation_mode])
        self.data[BsbDeviceProperties.DHW_MODE][PropertyType.VALUE] = BsbOperativeMode[operation_mode].value

    def set_water_heater_reduced_temperature(self, temperature: float):
        """Set water heater reduced temperature"""
        if len(self.data) == 0:
            self.update_state()
        current = self.water_heater_current_temperature
        if current is None:
            current = 0
        self._set_water_heater_temperature(current, temperature)

    async def async_set_water_heater_reduced_temperature(self, temperature: float):
        """Async set water heater temperature"""
        if len(self.data) == 0:
            await self.async_update_state()
        current = self.water_heater_current_temperature
        if current is None:
            current = 0
        await self._async_set_water_heater_temperature(current, temperature)

    def _set_water_heater_temperature(self, temperature: float, reduced: float):
        """Set water heater temperature"""
        self.api.set_bsb_temperature(self.gw, temperature, reduced, self.water_heater_target_temperature, self.water_heater_reduced_temperature)
        self.data[BsbDeviceProperties.DHW_COMF_TEMP][PropertyType.VALUE] = temperature
        self.data[BsbDeviceProperties.DHW_REDU_TEMP][PropertyType.VALUE] = reduced

    async def _async_set_water_heater_temperature(
        self, temperature: float, reduced: float
    ):
        """Async set water heater temperature"""
        await self.api.async_set_bsb_temperature(self.gw, temperature, reduced, self.water_heater_target_temperature, self.water_heater_reduced_temperature)
        self.data[BsbDeviceProperties.DHW_COMF_TEMP][PropertyType.VALUE] = temperature
        self.data[BsbDeviceProperties.DHW_REDU_TEMP][PropertyType.VALUE] = reduced

    def set_zone_mode(self, zone_mode: BsbZoneMode, zone: int):
        """Set zone mode"""
        self.api.set_bsb_zone_mode(self.gw, zone, zone_mode)

    async def async_set_zone_mode(self, zone_mode: BsbZoneMode, zone: int):
        """Async set zone mode"""
        await self.api.async_set_bsb_zone_mode(self.gw, zone, zone_mode)

    @property
    def outside_temp_value(self) -> str:
        """Get outside temperature value"""
        return self.data.get(BsbDeviceProperties.OUT_TEMP, 0)

    @property
    def outside_temp_unit(self) -> str:
        """Get outside temperature unit"""
        return "°C"

    def set_comfort_temp(self, temp: float, zone: int):
        """Set central heating comfort temp"""
        if len(self.data) == 0:
            self.update_state()
        reduced = self.get_reduced_temp_value(zone)
        self.api.set_bsb_zone_temperature(self.gw, zone, temp, reduced)
        self.get_zone_ch_comf_temp(zone)[PropertyType.VALUE] = temp

    async def async_set_comfort_temp(self, temp: float, zone: int):
        """Async set central heating comfort temp"""
        if len(self.data) == 0:
            await self.async_update_state()
        reduced = self.get_reduced_temp_value(zone)
        await self.api.async_set_bsb_zone_temperature(self.gw, zone, temp, reduced)
        self.get_zone_ch_comf_temp(zone)[PropertyType.VALUE] = temp

    def set_reduced_temp(self, temp: float, zone: int):
        """Set central heating reduced temp"""
        if len(self.data) == 0:
            self.update_state()
        comfort = self.get_comfort_temp_value(zone)
        self.api.set_bsb_zone_temperature(self.gw, zone, comfort, temp)
        self.get_zone_ch_red_temp(zone)[PropertyType.VALUE] = temp

    async def async_set_reduced_temp(self, temp: float, zone: int):
        """Async set central heating reduced temp"""
        if len(self.data) == 0:
            await self.async_update_state()
        comfort = self.get_comfort_temp_value(zone)
        await self.api.async_set_bsb_zone_temperature(self.gw, zone, comfort, temp)
        self.get_zone_ch_red_temp(zone)[PropertyType.VALUE] = temp
